@@ -6,6 +6,7 @@ from decimal import Decimal
 from autotrader.brokers.trading import Order
 from datetime import datetime, timedelta, timezone
 from autotrader.brokers.broker_utils import OrderBook
+import sys
 
 
 class AutoData:
@@ -26,7 +27,6 @@ class AutoData:
         data_config: dict = None,
         allow_dancing_bears: bool = False,
         home_currency: str = None,
-        *args,
         **kwargs,
     ) -> None:
         """Instantiates AutoData.
@@ -35,20 +35,26 @@ class AutoData:
         ----------
         data_config : dict, optional
             The configuration dictionary for the data source to be used. This
-            is created automatically in autotrader.utilities.get_config. The
-            default is None.
+            is created automatically in autotrader.utilities.get_data_config.
+            The default is None.
         allow_dancing_bears : bool, optional
             A flag to allow incomplete bars to be returned in the data. The
             default is False.
         home_currency : str, optional
             The home currency to use when fetching quote data. The default
             is None.
+        kwargs : optional
 
         Returns
         -------
         None
             AutoData will be instantiated and ready to fetch price data.
         """
+        # Merge kwargs and data_config
+        if data_config is None and kwargs is not None:
+            data_config = {}
+        for key, item in kwargs.items():
+            data_config.setdefault(key, item)
 
         def configure_local_feed(data_config):
             """Configures the attributes for a local data feed."""
@@ -69,17 +75,16 @@ class AutoData:
         else:
             self._feed = data_config["data_source"].lower()
 
-            # TODO - add KeyError handling
             if data_config["data_source"].lower() == "oanda":
                 API = data_config["API"]
                 ACCESS_TOKEN = data_config["ACCESS_TOKEN"]
+                self.ACCOUNT_ID = data_config["ACCOUNT_ID"]
                 port = data_config["PORT"]
 
                 try:
                     import v20
 
                     self.api = v20.Context(hostname=API, token=ACCESS_TOKEN, port=port)
-                    self.ACCOUNT_ID = data_config["ACCOUNT_ID"]
                 except ImportError:
                     raise Exception("Please install v20 to use the Oanda data feed.")
 
@@ -111,7 +116,25 @@ class AutoData:
                     import ccxt
 
                     self._ccxt_exchange = data_config["exchange"]
-                    self.api = getattr(ccxt, data_config["exchange"])()
+
+                    # Check if exchange options were provided
+                    if "config" in data_config:
+                        # Use config dict provided
+                        ccxt_config = data_config["config"]
+                    elif "secret" in data_config and "api_key" in data_config:
+                        ccxt_config = {
+                            "secret": data_config["secret"],
+                            "apiKey": data_config["api_key"],
+                        }
+                    else:
+                        # Create empty config dict
+                        ccxt_config = {}
+
+                    # Create CCXT instance
+                    self.api = getattr(ccxt, data_config["exchange"])(
+                        config=ccxt_config
+                    )
+
                 except ImportError:
                     raise Exception("Please install ccxt to use the CCXT data feed.")
 
@@ -985,6 +1008,7 @@ class AutoData:
         count: int = None,
         start_time: datetime = None,
         end_time: datetime = None,
+        **kwargs,
     ) -> pd.DataFrame:
         """Retrieves historical price data of a instrument from an exchange
         instance of the CCXT package.
@@ -1032,7 +1056,11 @@ class AutoData:
             data = []
             while start_ts <= end_ts:
                 raw_data = self.api.fetchOHLCV(
-                    instrument, timeframe=granularity, since=start_ts, limit=count
+                    instrument,
+                    timeframe=granularity,
+                    since=start_ts,
+                    limit=count,
+                    params=kwargs,
                 )
                 # Append data
                 data += raw_data
@@ -1049,7 +1077,7 @@ class AutoData:
             if start_time is None and end_time is None:
                 # Fetch N most recent candles
                 raw_data = self.api.fetchOHLCV(
-                    instrument, timeframe=granularity, limit=count
+                    instrument, timeframe=granularity, limit=count, params=kwargs
                 )
             elif start_time is not None and end_time is None:
                 # Fetch N candles since start_time
@@ -1057,7 +1085,11 @@ class AutoData:
                     None if start_time is None else int(start_time.timestamp() * 1000)
                 )
                 raw_data = self.api.fetchOHLCV(
-                    instrument, timeframe=granularity, since=start_ts, limit=count
+                    instrument,
+                    timeframe=granularity,
+                    since=start_ts,
+                    limit=count,
+                    params=kwargs,
                 )
             elif end_time is not None and start_time is None:
                 raise Exception(
@@ -1127,13 +1159,24 @@ class AutoData:
 
         return trades
 
+    def _ccxt_funding_rate(self, instrument: str):
+        """Returns the current funding rate."""
+        response = self.api.fetchFundingRate(instrument)
+
+        fr_dict = {
+            "symbol": instrument,
+            "rate": response["fundingRate"],
+            "time": response["fundingDatetime"],
+        }
+        return fr_dict
+
     def _ccxt_funding_history(
         self,
         instrument: str,
         count: int = None,
         start_time: datetime = None,
         end_time: datetime = None,
-        **kwargs,
+        params: dict = {},
     ):
         """Fetches the funding rate history."""
 
@@ -1155,7 +1198,7 @@ class AutoData:
             rate_hist = pd.DataFrame()
             while start_ts <= end_ts:
                 response = self.api.fetchFundingRateHistory(
-                    symbol=instrument, since=start_ts, limit=count, params=kwargs
+                    symbol=instrument, since=start_ts, limit=count, params=params
                 )
 
                 # Append results
@@ -1174,7 +1217,7 @@ class AutoData:
             if start_time is None and end_time is None:
                 # Fetch N most recent candles
                 response = self.api.fetchFundingRateHistory(
-                    symbol=instrument, limit=count, params=kwargs
+                    symbol=instrument, limit=count, params=params
                 )
                 rate_hist = response2df(response)
             elif start_time is not None and end_time is None:
@@ -1183,7 +1226,7 @@ class AutoData:
                     None if start_time is None else int(start_time.timestamp() * 1000)
                 )
                 response = self.api.fetchFundingRateHistory(
-                    symbol=instrument, since=start_ts, limit=count, params=kwargs
+                    symbol=instrument, since=start_ts, limit=count, params=params
                 )
                 rate_hist = response2df(response)
             elif end_time is not None and start_time is None:
@@ -1199,7 +1242,8 @@ class AutoData:
                 rate_hist = fetch_between_dates()
             else:
                 response = self.api.fetchFundingRateHistory(
-                    symbol=instrument, limit=count, params=kwargs
+                    symbol=instrument,
+                    params=params,
                 )
                 rate_hist = response2df(response)
 
