@@ -92,6 +92,7 @@ class Broker:
         self._equity = 0  # Account equity (balance)
         self._floating_pnl = 0
         self._margin_available = 0
+        self._open_interest = 0
 
         self._leverage = 1  # The account leverage
         self._spread = 0  # The bid/ask spread
@@ -111,6 +112,7 @@ class Broker:
         self._NAV_hist = []
         self._equity_hist = []
         self._margin_hist = []
+        self._open_interest_hist = []
         self._time_hist = []
 
         # Last order and trade counts
@@ -250,8 +252,7 @@ class Broker:
         if autodata_config is not None:
             # Instantiate AutoData from config
             data_config = get_data_config(
-                global_config=autodata_config["global_config"],
-                feed=autodata_config["feed"],
+                **autodata_config,
             )
             self._autodata = AutoData(data_config, **autodata_config)
 
@@ -569,6 +570,7 @@ class Broker:
                 short_PL = 0
                 short_margin = 0
                 total_margin = 0
+                pnl = 0
                 last_price = None
                 trade_IDs = []
 
@@ -578,6 +580,7 @@ class Broker:
                         trade.last_price if trade.last_price is not None else last_price
                     )
                     total_margin += trade.margin_required
+                    pnl += trade.unrealised_PL
                     if trade.direction > 0:
                         # Long trade
                         long_units += trade.size
@@ -609,6 +612,7 @@ class Broker:
                     "net_position": net_position,
                     "net_exposure": net_exposure,
                     "last_price": last_price,
+                    "pnl": pnl,
                 }
 
                 # Create Position instance
@@ -617,7 +621,7 @@ class Broker:
                 # Append Position to open_positions dict
                 open_positions[instrument] = instrument_position
 
-        return open_positions
+        return open_positions.copy()
 
     def get_margin_available(self) -> float:
         """Returns the margin available on the account."""
@@ -892,6 +896,7 @@ class Broker:
         self._NAV_hist.append(self._NAV)
         self._equity_hist.append(self._equity)
         self._margin_hist.append(self._margin_available)
+        self._open_interest_hist.append(self._open_interest)
         self._time_hist.append(latest_time)
 
         # Save state
@@ -1253,8 +1258,7 @@ class Broker:
             else exit_price
         )
 
-        # Get open trades for instrument
-        open_trades = self.get_isolated_positions(order.instrument)
+        # Get current position in instrument
         position = self.get_positions(order.instrument)
 
         # Modify existing trades until there are no more units to reduce
@@ -1265,6 +1269,11 @@ class Broker:
         executed_sizes = []
         while units_to_reduce > 0:
             # There are units to be reduced
+            open_trades = self.get_isolated_positions(order.instrument)
+            # TODO - issue when a margin call happens during this process,
+            # and the open_trades dict is no longer truthful. Need to review
+            # margin call process, as reducing should not increase margin
+            # requirements
             for trade_id, trade in open_trades.items():
                 if trade.direction != order.direction:
                     # Reduce this trade
@@ -1517,8 +1526,12 @@ class Broker:
         self, instrument: str = None, latest_time: datetime = None
     ) -> None:
         """Updates the margin available in the account."""
+        # TODO - only update with instrument
+
         margin_used = 0
         floating_pnl = 0
+        open_interest = 0
+
         open_trades = self.get_isolated_positions()
         for trade_id, trade in open_trades.items():
             size = trade.size
@@ -1535,12 +1548,18 @@ class Broker:
             # Floating pnl
             floating_pnl += trade.unrealised_PL
 
+            # Open interest
+            open_interest += abs(trade.value)
+
         # Update unrealised PnL
         self._floating_pnl = floating_pnl
 
         # Update margin available
         self._margin_available = self._NAV - margin_used
 
+        # Update open interest
+        self._open_interest = open_interest
+
         # Check for margin call
         if (
             self._leverage > 1
