@@ -1,3 +1,4 @@
+from glob import glob
 import sys
 import yaml
 import pickle
@@ -43,24 +44,31 @@ def write_yaml(data: dict, filepath: str) -> None:
 
 
 def get_broker_config(
-    global_config: dict, broker: str, environment: str = "paper"
+    broker: str, global_config: dict = None, environment: str = "paper"
 ) -> dict:
     """Returns a broker configuration dictionary.
 
     Parameters
     ----------
-    global_config : dict
-        The global configuration dictionary.
     broker : str
         The name(s) of the broker/exchange. Specify multiple exchanges using
         comma separation.
-    environment : str
+    global_config : dict
+        The global configuration dictionary.
+    environment : str, optional
         The trading evironment ('demo' or 'real').
 
     """
     all_config = {}
     inputted_brokers = broker.lower().replace(" ", "").split(",")
 
+    # Check global_config
+    if global_config is None:
+        try:
+            global_config = read_yaml("config/keys.yaml")
+        except:
+            pass
+
     for broker in inputted_brokers:
         # Check for CCXT
         if broker.split(":")[0].lower() == "ccxt":
@@ -81,13 +89,34 @@ def get_broker_config(
 
         # Live trading
         if broker.lower() == "oanda":
-            api_key = "LIVE_API" if environment.lower() == "live" else "PRACTICE_API"
-            config = {
-                "API": global_config["OANDA"][api_key],
-                "ACCESS_TOKEN": global_config["OANDA"]["ACCESS_TOKEN"],
-                "ACCOUNT_ID": global_config["OANDA"]["DEFAULT_ACCOUNT_ID"],
-                "PORT": global_config["OANDA"]["PORT"],
-            }
+            api_key = "LIVE" if environment.lower() == "live" else "PRACTICE"
+            oanda_conf = global_config["OANDA"]
+
+            # Initialise config dict
+            config = {"PORT": oanda_conf["PORT"]}
+
+            # Unpack
+            if f"{api_key}_API" in oanda_conf:
+                config["API"] = oanda_conf[f"{api_key}_API"]
+            else:
+                raise Exception(
+                    f"Please define {api_key}_API in your "
+                    + f"account configuration for {environment} trading."
+                )
+
+            if f"{api_key}_ACCESS_TOKEN" in oanda_conf:
+                config["ACCESS_TOKEN"] = oanda_conf[f"{api_key}_ACCESS_TOKEN"]
+            else:
+                raise Exception(
+                    f"Please define {api_key}_ACCESS_TOKEN in "
+                    + f"your account configuration for {environment} trading."
+                )
+
+            config["ACCOUNT_ID"] = (
+                oanda_conf["DEFAULT_ACCOUNT_ID"]
+                if "custom_account_id" not in global_config
+                else global_config["custom_account_id"]
+            )
 
         elif broker.lower() == "ib":
             config = {
@@ -126,20 +155,30 @@ def get_broker_config(
 
         elif broker.lower() == "ccxt":
             try:
-                config = global_config[broker_key.upper()]
-                api_key = config["api_key"] if "api_key" in config else None
-                secret = config["secret"] if "secret" in config else None
+                config_data = global_config[broker_key.upper()]
+                api_key = config_data["api_key"] if "api_key" in config_data else None
+                secret = config_data["secret"] if "secret" in config_data else None
                 currency = (
-                    config["base_currency"] if "base_currency" in config else "USDT"
+                    config_data["base_currency"]
+                    if "base_currency" in config_data
+                    else "USDT"
                 )
                 sandbox_mode = False if environment.lower() == "live" else True
                 config = {
+                    "data_source": "ccxt",
                     "exchange": exchange,
                     "api_key": api_key,
                     "secret": secret,
                     "sandbox_mode": sandbox_mode,
                     "base_currency": currency,
                 }
+                other_args = {"options": {}, "password": None}
+                for key, default_val in other_args.items():
+                    if key in config_data:
+                        config[key] = config_data[key]
+                    else:
+                        config[key] = default_val
+
             except KeyError:
                 raise Exception(
                     "Using CCXT for trading requires authentication via "
@@ -167,7 +206,7 @@ def get_broker_config(
     return all_config
 
 
-def get_data_config(feed: str, global_config: dict = None) -> dict:
+def get_data_config(feed: str, global_config: dict = None, **kwargs) -> dict:
     """Returns a configuration dictionary for AutoData.
     Parameters
     ----------
@@ -189,6 +228,13 @@ def get_data_config(feed: str, global_config: dict = None) -> dict:
     if feed.lower() not in supported_feeds:
         raise Exception(f"Unsupported data feed: '{feed}'")
 
+    # Check global_config
+    if global_config is None:
+        try:
+            global_config = read_yaml("config/keys.yaml")
+        except:
+            pass
+
     # Check for required authentication
     auth_feeds = ["oanda", "ib"]
     if feed.lower() in auth_feeds and global_config is None:
@@ -201,9 +247,33 @@ def get_data_config(feed: str, global_config: dict = None) -> dict:
     config = {"data_source": feed.lower()}
 
     if feed.lower() == "oanda":
-        config["API"] = global_config["OANDA"]["LIVE_API"]
-        config["ACCESS_TOKEN"] = global_config["OANDA"]["ACCESS_TOKEN"]
-        config["PORT"] = global_config["OANDA"]["PORT"]
+        environment = kwargs["environment"] if "environment" in kwargs else "paper"
+        api_key = "LIVE" if environment.lower() == "live" else "PRACTICE"
+        oanda_conf = global_config["OANDA"]
+
+        # Unpack
+        if f"{api_key}_API" in oanda_conf:
+            config["API"] = oanda_conf[f"{api_key}_API"]
+        else:
+            raise Exception(
+                f"Please define {api_key}_API in your "
+                + f"account configuration for {environment} trading."
+            )
+
+        if f"{api_key}_ACCESS_TOKEN" in oanda_conf:
+            config["ACCESS_TOKEN"] = oanda_conf[f"{api_key}_ACCESS_TOKEN"]
+        else:
+            raise Exception(
+                f"Please define {api_key}_ACCESS_TOKEN in "
+                + f"your account configuration for {environment} trading."
+            )
+
+        config["PORT"] = oanda_conf["PORT"]
+        config["ACCOUNT_ID"] = (
+            oanda_conf["DEFAULT_ACCOUNT_ID"]
+            if "custom_account_id" not in global_config
+            else global_config["custom_account_id"]
+        )
 
     elif feed.lower() == "ib":
         config["host"] = (
@@ -488,19 +558,22 @@ class TradeAnalysis:
                     "NAV": broker._NAV_hist,
                     "equity": broker._equity_hist,
                     "margin": broker._margin_hist,
+                    "open_interest": broker._open_interest_hist,
                 },
                 index=broker._time_hist,
             )
 
+            # Remove duplicates
+            account_history = account_history[
+                ~account_history.index.duplicated(keep="last")
+            ]
+
             position_history = TradeAnalysis.create_position_history(
                 trade_history=trade_history,
                 account_history=account_history,
             )
 
             # Calculate drawdown
-            account_history = account_history[
-                ~account_history.index.duplicated(keep="last")
-            ]
             account_history["drawdown"] = (
                 account_history.NAV / account_history.NAV.cummax() - 1
             )
@@ -833,6 +906,12 @@ class TradeAnalysis:
         # All trades
         no_trades = len(self.trade_history)
         trade_results["no_trades"] = no_trades
+        trade_results["no_long_trades"] = len(
+            self.trade_history[self.trade_history["direction"] > 0]
+        )
+        trade_results["no_short_trades"] = len(
+            self.trade_history[self.trade_history["direction"] < 0]
+        )
 
         if no_trades > 0:
             # Initialise all_trades dict
@@ -860,9 +939,17 @@ class TradeAnalysis:
             longest_win_streak, longest_lose_streak = get_streaks(
                 self.isolated_position_history
             )
-            avg_trade_duration = np.nanmean(
-                self.isolated_position_history.trade_duration.values
-            )
+            try:
+                avg_trade_duration = np.nanmean(
+                    self.isolated_position_history.trade_duration.values
+                )
+                trade_results["all_trades"]["avg_trade_duration"] = str(
+                    timedelta(seconds=int(avg_trade_duration))
+                )
+            except TypeError:
+                # Position has not been closed yet
+                trade_results["all_trades"]["avg_trade_duration"] = None
+
             min_trade_duration = np.nanmin(
                 self.isolated_position_history.trade_duration.values
             )
@@ -878,63 +965,69 @@ class TradeAnalysis:
             trade_results["all_trades"]["win_rate"] = win_rate
             trade_results["all_trades"]["win_streak"] = longest_win_streak
             trade_results["all_trades"]["lose_streak"] = longest_lose_streak
-            trade_results["all_trades"]["longest_trade"] = str(
-                timedelta(seconds=int(max_trade_duration))
-            )
-            trade_results["all_trades"]["shortest_trade"] = str(
-                timedelta(seconds=int(min_trade_duration))
-            )
-            trade_results["all_trades"]["avg_trade_duration"] = str(
-                timedelta(seconds=int(avg_trade_duration))
-            )
+
+            if max_trade_duration is not None:
+                trade_results["all_trades"]["longest_trade"] = str(
+                    timedelta(seconds=int(max_trade_duration))
+                )
+            else:
+                trade_results["all_trades"]["longest_trade"] = str(None)
+
+            if min_trade_duration is not None:
+                trade_results["all_trades"]["shortest_trade"] = str(
+                    timedelta(seconds=int(min_trade_duration))
+                )
+            else:
+                trade_results["all_trades"]["shortest_trade"] = str(None)
+
             trade_results["all_trades"]["total_fees"] = total_fees
 
         # Cancelled orders
         trade_results["no_cancelled"] = len(self.cancelled_orders)
 
         # Long positions
-        long_trades = self.isolated_position_history[
+        long_positions = self.isolated_position_history[
             self.isolated_position_history["direction"] > 0
         ]
-        no_long = len(long_trades)
-        trade_results["long_trades"] = {}
-        trade_results["long_trades"]["no_trades"] = no_long
+        no_long = len(long_positions)
+        trade_results["long_positions"] = {}
+        trade_results["long_positions"]["total"] = no_long
         if no_long > 0:
-            long_wins = long_trades[long_trades.profit > 0]
+            long_wins = long_positions[long_positions.profit > 0]
             avg_long_win = np.mean(long_wins.profit)
             max_long_win = np.max(long_wins.profit)
-            long_loss = long_trades[long_trades.profit < 0]
+            long_loss = long_positions[long_positions.profit < 0]
             avg_long_loss = abs(np.mean(long_loss.profit))
             max_long_loss = abs(np.min(long_loss.profit))
-            long_wr = 100 * len(long_trades[long_trades.profit > 0]) / no_long
+            long_wr = 100 * len(long_positions[long_positions.profit > 0]) / no_long
 
-            trade_results["long_trades"]["avg_long_win"] = avg_long_win
-            trade_results["long_trades"]["max_long_win"] = max_long_win
-            trade_results["long_trades"]["avg_long_loss"] = avg_long_loss
-            trade_results["long_trades"]["max_long_loss"] = max_long_loss
-            trade_results["long_trades"]["long_wr"] = long_wr
+            trade_results["long_positions"]["avg_long_win"] = avg_long_win
+            trade_results["long_positions"]["max_long_win"] = max_long_win
+            trade_results["long_positions"]["avg_long_loss"] = avg_long_loss
+            trade_results["long_positions"]["max_long_loss"] = max_long_loss
+            trade_results["long_positions"]["long_wr"] = long_wr
 
         # Short positions
-        short_trades = self.isolated_position_history[
+        short_positions = self.isolated_position_history[
             self.isolated_position_history["direction"] < 0
         ]
-        no_short = len(short_trades)
-        trade_results["short_trades"] = {}
-        trade_results["short_trades"]["no_trades"] = no_short
+        no_short = len(short_positions)
+        trade_results["short_positions"] = {}
+        trade_results["short_positions"]["total"] = no_short
         if no_short > 0:
-            short_wins = short_trades[short_trades.profit > 0]
+            short_wins = short_positions[short_positions.profit > 0]
             avg_short_win = np.mean(short_wins.profit)
             max_short_win = np.max(short_wins.profit)
-            short_loss = short_trades[short_trades.profit < 0]
+            short_loss = short_positions[short_positions.profit < 0]
             avg_short_loss = abs(np.mean(short_loss.profit))
             max_short_loss = abs(np.min(short_loss.profit))
-            short_wr = 100 * len(short_trades[short_trades.profit > 0]) / no_short
+            short_wr = 100 * len(short_positions[short_positions.profit > 0]) / no_short
 
-            trade_results["short_trades"]["avg_short_win"] = avg_short_win
-            trade_results["short_trades"]["max_short_win"] = max_short_win
-            trade_results["short_trades"]["avg_short_loss"] = avg_short_loss
-            trade_results["short_trades"]["max_short_loss"] = max_short_loss
-            trade_results["short_trades"]["short_wr"] = short_wr
+            trade_results["short_positions"]["avg_short_win"] = avg_short_win
+            trade_results["short_positions"]["max_short_win"] = max_short_win
+            trade_results["short_positions"]["avg_short_loss"] = avg_short_loss
+            trade_results["short_positions"]["max_short_loss"] = max_short_loss
+            trade_results["short_positions"]["short_wr"] = short_wr
 
         return trade_results
 
